@@ -13,12 +13,9 @@ from cursor import Cursor
 from encodedsession import EncodedSession
 from crypt import ClientPassword, RC4Cipher
 from util import getCloudEntry
-from session import SessionException
-from exception import ProgrammingError
 
 import time
 import string
-import protocol
 
 # http://www.python.org/dev/peps/pep-0249
 
@@ -26,7 +23,8 @@ apilevel = "2.0"
 threadsafety = 1
 paramstyle = "qmark"
 #schema='user', auto_commit=False
-def connect(database, host, user, password, options = None ):
+
+def connect(database, host, user, password, options=None):
     """Creates a connection object.
     Arguments:
     database -- Name of the database to access.
@@ -36,7 +34,12 @@ def connect(database, host, user, password, options = None ):
     options -- A dictionary of NuoDB connection options
         Some common options include:
         "schema"
-        
+
+    @type database str
+    @type host str
+    @type user str
+    @type password str
+    @type options dict[str,str]
     """
     return Connection(database, host, user, password, options)
 
@@ -59,7 +62,7 @@ class Connection(object):
     auto_commit (getter) -- Gets the value of auto_commit from the database.
     auto_commit (setter) -- Sets the value of auto_commit on the database.
     """
-    
+
     from exception import Warning, Error, InterfaceError, DatabaseError, DataError, \
             OperationalError, IntegrityError, InternalError, \
             ProgrammingError, NotSupportedError
@@ -78,6 +81,12 @@ class Connection(object):
         
         Returns:
         a Connection instance
+
+        @type dbName str
+        @type broker str
+        @type username str
+        @type password str
+        @type options dict[str,str]
         """
         (host, port) = getCloudEntry(broker, dbName)
         self.__session = EncodedSession(host, port)
@@ -89,32 +98,15 @@ class Connection(object):
         if options:
             parameters.update(options)
 
-        self.__session.putMessageId(protocol.OPENDATABASE).putInt(protocol.EXECUTEPREPAREDUPDATE).putString(dbName).putInt(len(parameters))
-        for (k, v) in parameters.iteritems():
-            self.__session.putString(k).putString(v)
-        self.__session.putNull().putString(cp.genClientKey())
-
-        self.__session.exchangeMessages()
-
-        version = self.__session.getInt()
-        serverKey = self.__session.getString()
-        salt = self.__session.getString()
+        version, serverKey, salt = self.__session.open_database(dbName, parameters, cp)
 
         sessionKey = cp.computeSessionKey(string.upper(username), password, salt, serverKey)
         self.__session.setCiphers(RC4Cipher(sessionKey), RC4Cipher(sessionKey))
 
-        # check auth
-        try:
-            self.__session.putMessageId(protocol.AUTHENTICATION).putString('Success!')
-            self.__session.exchangeMessages()
-        except SessionException:
-            raise ProgrammingError('Invalid database username or password')
-            
-        
-        # set auto commit to false by default
-        self.__session.putMessageId(protocol.SETAUTOCOMMIT).putInt(0)
-        
-        self.__session.exchangeMessages(False)
+        self.__session.check_auth()
+
+        # set auto commit to false by default per PEP
+        self.__session.set_autocommit(0)
 
     def testConnection(self):
         """Tests to ensure the connection was properly established.
@@ -130,42 +122,25 @@ class Connection(object):
         Returns:
         None
         """
-        # Create a statement handle
-        self.__session.putMessageId(protocol.CREATE)
-        self.__session.exchangeMessages()
-        handle = self.__session.getInt()
-
-        # Use handle to query dual
-        self.__session.putMessageId(protocol.EXECUTEQUERY).putInt(handle).putString('select 1 as one from dual')
-        self.__session.exchangeMessages()
-
-        rsHandle = self.__session.getInt()
-        count = self.__session.getInt()
-        colname = self.__session.getString()
-        result = self.__session.getInt()
-        fieldValue = self.__session.getInt()
-        r2 = self.__session.getInt()
+        self.__session.test_connection()
 
     @property
     def auto_commit(self):
         """Gets the value of auto_commit from the database."""
         self._check_closed()
-        self.__session.putMessageId(protocol.GETAUTOCOMMIT)
-        self.__session.exchangeMessages()
-        return self.__session.getValue()
+        return self.__session.get_autocommit()
     
     @auto_commit.setter
     def auto_commit(self, value):
         """Sets the value of auto_commit on the database."""
         self._check_closed()
-        self.__session.putMessageId(protocol.SETAUTOCOMMIT).putInt(value)
-        self.__session.exchangeMessages(False)
+        self.__session.set_autocommit(value)
 
     def close(self):
         """Closes the connection with the host."""
         self._check_closed()
-        self.__session.putMessageId(protocol.CLOSE)
-        self.__session.exchangeMessages()
+        self.__session.send_close()
+        #TODO: This sends a close request, but doesn't force close the socket
         self.__session.closed = True
 
     def _check_closed(self):
@@ -176,18 +151,14 @@ class Connection(object):
     def commit(self):
         """Sends a message to the host to commit transaction."""
         self._check_closed()
-        self.__session.putMessageId(protocol.COMMITTRANSACTION)
-        self.__session.exchangeMessages()
-        self._trans_id = self.__session.getValue()
+        self._trans_id = self.__session.send_commit()
 
     def rollback(self):
         """Sends a message to the host to rollback uncommitted changes."""
         self._check_closed()
-        self.__session.putMessageId(protocol.ROLLBACKTRANSACTION)
-        self.__session.exchangeMessages()
+        self.__session.send_rollback()
 
-    def cursor(self):
+    def cursor(self, prepared_statement_cache_size=50):
         """Return a new Cursor object using the connection."""
         self._check_closed()
-        return Cursor(self.__session)
-    
+        return Cursor(self.__session, prepared_statement_cache_size)
